#include "abstractobj.h"

AbstractObj::AbstractObj(AbstractObj::ObjType objType)
{
    m_objType = objType;
    m_parentObj = 0;
}

void AbstractObj::appendProperty(AbstractProperty *property)
{
    m_properties.append(property);
}

QVariant AbstractObj::displayRoleDataForPropertyAt(int pos) const
{
    if (pos>=0 && pos<m_properties.count())
        return m_properties.at(pos)->displayRoleData();
    else
        return QVariant();
}

QVariant AbstractObj::displayRoleDataAt(int pos) const
{
    if (pos==0)
        return commandNameForDisplayRoleData();
    else
        return displayRoleDataForPropertyAt(pos-1);
}

QVariant AbstractObj::editRoleDataAt(int pos) const
{
    if (pos == 0)
        return QVariant(); // Command name is not editable
    else
    {
        return m_properties.at(pos-1)->editRoleData();
    }
}

bool AbstractObj::setEditRoleDataForPropertyAt(int pos, QVariant data)
{
    return m_properties.at(pos)->setEditRoleData(data);
}

int AbstractObj::propertiesCount() const
{
    return m_properties.count();
}

AbstractProperty *AbstractObj::propertyAt(int pos)
{
    return m_properties.at(pos);
}

void AbstractObj::setFromObjDomElement(const QDomElement &domElement)
{
    for (int i=0; i<m_properties.count(); i++)
        m_properties.at(i)->setFromObjDomElement(domElement);
}

QDomElement AbstractObj::domElement(QDomDocument &xmlDocument) const
{
    QDomElement element = xmlDocument.createElement("object");
    element.setAttribute("type", typeNameForDomElement());

    for (int i=0; i<m_properties.count(); i++)
        element.appendChild(m_properties.at(i)->domElement(xmlDocument));

    return element;
}

QString AbstractObj::standardInnerCommandString() const
{
    QString requiredCommandString;
    QString optionalCommandString;
    for (int i=0; i<m_properties.count(); i++)
    {
        if (m_properties.at(i)->isRequired())
        {
            QString string = m_properties.at(i)->commandString();
            if (!string.isEmpty())
                requiredCommandString += " " + string;
        }
        else
        {
            QString string = m_properties.at(i)->commandString();
            if (!string.isEmpty())
                optionalCommandString += " " + string;
        }
    }
    if (optionalCommandString.isEmpty())
        return requiredCommandString;
    else
        return requiredCommandString + "," + optionalCommandString;
}

int AbstractObj::row() const
{
    if (m_parentObj)
        return m_parentObj->m_childObjs.indexOf(const_cast<AbstractObj*>(this));
    else
        return -1;
}

AbstractObj *AbstractObj::objAt(int row) const
{
    if (row>=0 && row<m_childObjs.count())
        return m_childObjs.at(row);
    else
        return 0;
}

AbstractObj *AbstractObj::parentObj()
{
    return m_parentObj;
}

void AbstractObj::appendObj(AbstractObj *obj)
{
    m_childObjs.append(obj);
    obj->m_parentObj = this;
}

void AbstractObj::insertObj(AbstractObj *obj, int pos)
{
    m_childObjs.insert(pos, obj);
    obj->m_parentObj = this;
}

void AbstractObj::swapObjects(int pos1, int pos2)
{
    m_childObjs.swap(pos1, pos2);
}

void AbstractObj::deleteObjAt(int pos)
{
    if (pos>=0 && pos<m_childObjs.count())
    {
        delete objAt(pos);
        m_childObjs.removeAt(pos);
    }
}

void AbstractObj::deleteChildren()
{
    qDeleteAll(m_childObjs);
    m_childObjs.clear();
}

bool AbstractObj::isControlObj() const
{
    switch (m_objType)
    {
        case AbstractObj::CheckBox:
        case AbstractObj::Radio:
        case AbstractObj::Spinner:
        case AbstractObj::Edit:
        case AbstractObj::VarList:
        case AbstractObj::VarName:
        case AbstractObj::File:
        case AbstractObj::ListBox:
        case AbstractObj::ComboBox:
        case AbstractObj::Button:
        case AbstractObj::Text:
        case AbstractObj::TextBox:
        case AbstractObj::GroupBox:
        case AbstractObj::Frame:
        case AbstractObj::Color:
        case AbstractObj::Exp:
            return true;
            break;
        default:
            return false;
            break;
    }
}

bool AbstractObj::hasAtLeastOneChildControlObjBeforeRow(int row) const
{
    for (int i=0; i<row; i++)
    {
        if (m_childObjs.at(i)->isControlObj())
            return true;
    }
    return false;
}

