#include "debugreportmodel.h"

DebugReportModel::DebugReportModel(Document *document, QObject *parent) :
    QAbstractItemModel(parent)
{
    this->document = document;
}

DebugReportModel::~DebugReportModel()
{
    qDeleteAll(debugReportItems);
}

int DebugReportModel::rowCount(const QModelIndex &parent) const
{
    return debugReportItems.count();
}

int	DebugReportModel::columnCount(const QModelIndex & parent) const
{
    return 1;
}

QVariant DebugReportModel::data(const QModelIndex & index, int role) const
{
    if (!index.isValid())
        return QVariant();

    if (index.row() >= debugReportItems.count())
        return QVariant();

    switch (index.column())
    {
        case 0:
            switch (role)
            {
                case Qt::DisplayRole:
                    return debugReportItems.at(index.row())->issue();
                    break;
                case Qt::EditRole:
                    return debugReportItems.at(index.row())->moreOnIssue();
                    break;
                default:
                    return QVariant();
                    break;
            }
            break;
        default:
            return QVariant();
            break;
    }
}

QModelIndex	DebugReportModel::parent(const QModelIndex & index) const
{
    Q_UNUSED(index)
    return QModelIndex();
}

QModelIndex DebugReportModel::index(int row, int column, const QModelIndex &parent) const
{
    if (!hasIndex(row, column, parent))
        return QModelIndex();

    return createIndex(row, column, debugReportItems.at(row));
}

void DebugReportModel::appendDebugReportItem(DebugReportItem *debugReportItem)
{
    int position = rowCount(QModelIndex());
    beginInsertRows(QModelIndex(), position, position);
    debugReportItems.append(debugReportItem);
    endInsertRows();
}

void DebugReportModel::clear()
{
    int count = rowCount(QModelIndex());
    if (count == 0) return;

    beginRemoveRows(QModelIndex(), 0,  count-1);
    qDeleteAll(debugReportItems);
    debugReportItems.clear();
    endRemoveRows();
}

void DebugReportModel::updateReport()
{
    clear();

    debugVersion();
    debugPosition();
    debugUniqueNames();
    debugAtLeastOneDialogExist();
}

QString DebugReportModel::objName(AbstractObj *obj)
{
    if (!obj->hasName()) return "";

    return obj->propertyAt(0)->value();
}

void DebugReportModel::debugVersion()
{
    int versionObjCount = 0;
    int versionPosition = -1;
    for (int i=0; i<document->objCount(); i++)
    {
        AbstractObj *obj = document->objAt(i);
        if (obj->objType() == AbstractObj::Version)
        {
            if (versionPosition == -1)
                versionPosition = i;
            versionObjCount++;
        }
    }

    if (versionObjCount == 0)
    {
        DebugReportItem *d = new DebugReportItem;
        d->setIssue("No \"VERSION\" object was defined");
        d->setMoreOnIssue("You must define one and only one \"VERSION\" object");
        appendDebugReportItem(d);
    }

    if (versionObjCount > 1)
    {
        DebugReportItem *d = new DebugReportItem;
        d->setIssue("More than one \"VERSION\" object was defined");
        d->setMoreOnIssue("You must define one and only one \"VERSION\" object");
        appendDebugReportItem(d);
    }

    if (versionObjCount>0 && versionPosition != 0)
    {
        DebugReportItem *d = new DebugReportItem;
        d->setIssue("A \"VERSION\" object was not defined in the very first line of the dialog code");
        d->setMoreOnIssue("The \"VERSION\" object must be the very first object (but it can be preceded by comments)");
        appendDebugReportItem(d);
    }
}

void DebugReportModel::debugPosition()
{
    int positionObjCount = 0;
    for (int i=0; i<document->objCount(); i++)
    {
        AbstractObj *obj = document->objAt(i);
        if (obj->objType() == AbstractObj::Position)
            positionObjCount++;
    }

    if (positionObjCount == 0)
    {
        DebugReportItem *d = new DebugReportItem;
        d->setIssue("No \"POSITION\" object was defined");
        d->setMoreOnIssue("You must define one and only one \"POSITION\" object");
        appendDebugReportItem(d);
    }

    if (positionObjCount > 1)
    {
        DebugReportItem *d = new DebugReportItem;
        d->setIssue("More than one \"POSITION\" object was defined");
        d->setMoreOnIssue("You must define one and only one \"POSITION\" object");
        appendDebugReportItem(d);
    }

}

void DebugReportModel::debugUniqueNames()
{
    // Check in root
    QStringList namesInRoot;
    for (int i=0; i<document->objCount(); i++)
    {
        AbstractObj *obj = document->objAt(i);
        if (obj->hasName())
            namesInRoot.append(objName(obj));
    }
    QStringList uniqueNamesInRoot = namesInRoot;
    uniqueNamesInRoot.removeDuplicates();
    for (int i=0; i<uniqueNamesInRoot.count(); i++)
    {
        QString name = uniqueNamesInRoot.at(i);
        int nObjWithTheSameName = namesInRoot.count(name);
        if (nObjWithTheSameName>1)
        {
            DebugReportItem *d = new DebugReportItem;
            QString issue = QString("In the root: there are %1 objects with the same name \"%2\"")
                    .arg(nObjWithTheSameName)
                    .arg(name);
            d->setIssue(issue);
            d->setMoreOnIssue("Every object in the root must have an unique name. Rename objects assigning them an unique name");
            appendDebugReportItem(d);
        }
    }

    // Check in dialogs
    for (int i=0; i<document->objCount(); i++)
    {
        AbstractObj *obj = document->objAt(i);
        AbstractObj::ObjType objType = obj->objType();
        switch (objType)
        {
            case AbstractObj::Dialog:
                {
                QString nameOfDialog = objName(obj);
                QStringList namesInDialog;
                for (int j=0; j<obj->objCount(); j++)
                {
                    if (obj->objAt(j)->hasName())
                        namesInDialog.append(objName(obj->objAt(j)));
                }
                QStringList uniqueNamesInDialog = namesInDialog;
                uniqueNamesInDialog.removeDuplicates();
                for (int k=0; k<uniqueNamesInDialog.count(); k++)
                {
                    QString name = uniqueNamesInDialog.at(k);
                    int nObjWithTheSameName = namesInDialog.count(name);
                    if (nObjWithTheSameName>1)
                    {
                        DebugReportItem *d = new DebugReportItem;
                        QString issue = QString("In dialog \"%1\": there are %2 objects with the same name \"%3\"")
                                .arg(nameOfDialog)
                                .arg(nObjWithTheSameName)
                                .arg(name);
                        d->setIssue(issue);
                        d->setMoreOnIssue("Every object in a dialog must have an unique name. Rename objects assigning them an unique name");
                        appendDebugReportItem(d);
                    }
                }
                }
                break;
            default:
                break;
        }
    }
}

void DebugReportModel::debugAtLeastOneDialogExist()
{
    int dialogCount = 0;
    for (int i=0; i<document->objCount(); i++)
    {
        AbstractObj *obj = document->objAt(i);
        if (obj->objType() == AbstractObj::Dialog)
            dialogCount++;
    }

    if (dialogCount == 0)
    {
        DebugReportItem *d = new DebugReportItem;
        d->setIssue("No dialog was defined");
        d->setMoreOnIssue("At least one dialog should be defined");
        appendDebugReportItem(d);
    }

    if (dialogCount > 15)
    {
        DebugReportItem *d = new DebugReportItem;
        d->setIssue("More than 15 dialogs were defined");
        d->setMoreOnIssue("Stata can't manage more than 15 dialogs. It's probable that this limit will be removed in future versions of Stata. Please try with you Stata");
        appendDebugReportItem(d);
    }
}
