#include "geometrydatasetter.h"
#include <QHBoxLayout>
#include <QMenu>
#include <QDebug>

GeometryDataSetter::GeometryDataSetter(const QStringList &userDefinedNameList, QWidget *parent)
    : AbstractSetter(parent)
{
    // Layout
    QHBoxLayout *layout = new QHBoxLayout;
    layout->setMargin(0);
    layout->setContentsMargins(0, 0, 0, 0);
    layout->setSpacing(0);
    setLayout(layout);

    // Menu
    QMenu *menu = new QMenu(this);

    // User defined list menu
    QMenu *userDefinedNameMenu = new QMenu("User defined name");
    for (int i=0; i<userDefinedNameList.count(); i++)
    {
        QAction *action = new QAction(userDefinedNameList.at(i), userDefinedNameMenu);
        action->setData(userDefinedNameList.at(i));
        userDefinedNameMenu->addAction(action);
        connect(action, SIGNAL(triggered()), this, SLOT(onUserDefinedNameMenuItemTriggered()));
    }
    if (userDefinedNameList.count() == 0) userDefinedNameMenu->setDisabled(true);
    menu->addMenu(userDefinedNameMenu);

    // Clear action
    QAction *clearAction = new QAction("Clear", this);
    menu->addAction(clearAction);
    connect(clearAction, SIGNAL(triggered()), this, SLOT(onClearActionsTriggered()));

    // Spin box
    spinBox = new QSpinBox;
    spinBox->setMinimum(0);
    spinBox->setMaximum(9999);
    layout->addWidget(spinBox);

    // Line Edit
    lineEdit = new QLineEdit;
    lineEdit->setMaxLength(32);
    QRegExp regExp("[^\\d][\\w]+");
    QRegExpValidator *regExpValidator = new QRegExpValidator(regExp, lineEdit);
    lineEdit->setValidator(regExpValidator);
    lineEdit->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    layout->addWidget(lineEdit);

    // Button
    button = new QPushButton;
    button->setMenu(menu);
    button->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
    button->setMinimumWidth(20);
    button->setMaximumWidth(20);
    layout->addWidget(button);

    // Combo box
    comboBox = new QComboBox;
        // No extra code
        comboBox->addItem("No extra code");
        // Default value
        comboBox->addItem("Default value");
        // Previous value
        comboBox->addItem("Previous value");
        // Positive offset
        comboBox->addItem("Positive offset");
        // Negative offset
        comboBox->addItem("Negative offset");
        // Name
        comboBox->addItem("Name");
    comboBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    layout->addWidget(comboBox);

    // Connections
    connect(comboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(onCurrentComboBoxItemChanged(int)));
}

void GeometryDataSetter::setGeometryData(const QString &data)
{
    // Data should not be empty
    if (data.isEmpty())
    {
        spinBox->hide();
        lineEdit->setText(data);
        lineEdit->show();
        button->show();
        comboBox->setCurrentIndex(5);
        comboBox->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
        return;
    }

    bool isNumeric;
    int numericValue = data.toInt(&isNumeric);

    if (isNumeric && data.at(0) != '+' && data.at(0) != '-') // No extra code
    {
        spinBox->setValue(numericValue);
        spinBox->setPrefix("");
        spinBox->show();
        lineEdit->hide();
        button->hide();
        comboBox->setCurrentIndex(0);
        comboBox->show();
        comboBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    }
    else if (data == ".") // Default value
    {
        spinBox->hide();
        lineEdit->hide();
        comboBox->setCurrentIndex(1);
        button->hide();
        comboBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    }
    else if (data == "@") // Previous value
    {
        spinBox->hide();
        lineEdit->hide();
        button->hide();
        comboBox->setCurrentIndex(2);
        comboBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    }
    else if (data.at(0) == '+') // Positive offset
    {
        spinBox->setPrefix("+");
        QString implicitValue = data.mid(1);
        spinBox->setValue(implicitValue.toInt());
        spinBox->show();
        lineEdit->hide();
        button->hide();
        comboBox->setCurrentIndex(3);
        comboBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    }
    else if (data.at(0) == '-') // Negative offset
    {
        spinBox->setPrefix("-");
        QString implicitValue = data.mid(1);
        spinBox->setValue(implicitValue.toInt());
        spinBox->show();
        lineEdit->hide();
        button->hide();
        comboBox->setCurrentIndex(4);
        comboBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    }
    else // Name
    {
        spinBox->hide();
        lineEdit->setText(data);
        lineEdit->show();
        button->show();
        comboBox->setCurrentIndex(5);
        comboBox->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    }
}

void GeometryDataSetter::onCurrentComboBoxItemChanged(int pos)
{
    switch (pos)
    {
        case 0: // No extra code
            spinBox->setPrefix("");
            spinBox->show();
            lineEdit->hide();
            button->hide();
            comboBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
            break;
        case 1: // Default value
            spinBox->hide();
            lineEdit->hide();
            button->hide();
            comboBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
            break;
        case 2: // Previous value
            spinBox->hide();
            lineEdit->hide();
            button->hide();
            comboBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
            break;
        case 3: // Positive offset
            spinBox->setPrefix("+");
            spinBox->show();
            lineEdit->hide();
            button->hide();
            comboBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
            break;
        case 4: // Negative offset
            spinBox->setPrefix("-");
            spinBox->show();
            lineEdit->hide();
            button->hide();
            comboBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
            break;
        case 5: // Name
            spinBox->hide();
            lineEdit->show();
            button->show();
            comboBox->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
            break;
        default:
            break;
    }
}

QString GeometryDataSetter::geometryData() const
{
    switch (comboBox->currentIndex())
    {
        case 0: // No extra code
            return QString("%1").arg(spinBox->value());
            break;
        case 1: // Default value
            return ".";
            break;
        case 2: // Previous value
            return "@";
            break;
        case 3: // Positive offset
            return QString("+%1").arg(spinBox->value());
            break;
        case 4: // Negative offset
            return QString("-%1").arg(spinBox->value());
            break;
        case 5: // Name
            return lineEdit->text();
            break;
        default:
            return QString();
            break;
    }
}

void GeometryDataSetter::onUserDefinedNameMenuItemTriggered()
{
    QAction *action = qobject_cast<QAction *>(sender());
    lineEdit->setText(action->text());
}

void GeometryDataSetter::onClearActionsTriggered()
{
    lineEdit->clear();
}

