#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QtGui>

// Document
#include "document.h"

// Model
#include "model.h"

// Object list proxy model and its GUI
#include "objectlistproxymodel.h"
#include "objectlistview.h"
#include "objectlistdockwidget.h"

// Object properties proxy model and its GUI
#include "objectpropertiesproxymodel.h"
#include "objectpropertiesview.h"
#include "objectpropertiesdockwidget.h"

// Code view
#include "codeview.h"
#include "codeviewdockwidget.h"

// Object box
#include "objectboxdockwidget.h"

// Canvas
#include "canvas.h"

// Stata log view
#include "statalogview.h"

// Help contents view
#include "helpcontentsview.h"

// Visua plugin interface
#include "visuaplugininterface.h"

// Debugging
#include "debugreportwindow.h"

class QMenu;

class MainWindow : public QMainWindow
{
Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);

private:

    // Document
    Document *document;

    // Model
    Model *model;

    // Object list proxy model and its GUI
    ObjectListProxyModel *objectListProxyModel;
    ObjectListDockWidget *objectListDockWidget;
    ObjectListView *objectListView;

    // Object properties proxy model and its GUI
    ObjectPropertiesProxyModel *objectPropertiesProxyModel;
    ObjectPropertiesView *objectPropertiesView;
    ObjectPropertiesDockWidget *objectPropertiesDockWidget;

    // Code view
    CodeView *codeView;
    CodeViewDockWidget *codeViewDockWidget;

    // Object box
    ObjectBoxDockWidget *objectBoxDockWidget;

    // Canvas
    Canvas *canvas;

    // Toolbar
    QToolBar *toolBar;

    // Stata log view
    StataLogView *stataLogView;

    // Help contents view
    HelpContentsView *helpContentsView;
    QRect helpContentsViewGeometry;

    // Debugging
    DebugReportWindow *debugReportWindow;

    // Other functions
    void setDummyModel();
    void createActions();
    void createMenus();
    void createToolBar();
    QString strippedFileName(const QString &fullFileName);
    void setCurrentFile(const QString &fileName);
    bool okToContinue();
    bool loadVisuaFile(const QString &fileName);
    bool saveVisuaFile(const QString &fileName);
    void writeSettings();
    void readSettings();
    void loadPlugins();
    void createNewFromTemplateActions();

    // Other data members
    QString currentFile;

    // Actions
    QAction *newFileAction;
    QAction *openFileAction;
    QAction *saveFileAction;
    QAction *saveFileAsAction;
    QAction *exportAction;
    QAction *exitAction;
    QAction *testDialogInStataAction;
    QAction *viewLastStataLogAction;
    QAction *stataPath;
    QAction *debugAction;
    QAction *viewLastDebugReportAction;
    QAction *viewObjectListAction;
    QAction *viewObjectPropertiesAction;
    QAction *viewCodeAction;
    QAction *viewObjectBoxAction;
    QAction *helpContentsAction;
    QAction *helpDialogProgramming;
    QAction *helpDialogProgrammingFAQsA1;
    QAction *helpDialogProgrammingFAQsA2;
    QAction *helpDialogProgrammingFAQsA3;
    QAction *helpDialogProgrammingFAQsA4;
    QAction *helpDialogProgrammingFAQsB1;
    QAction *helpDialogProgrammingFAQsB2;
    QAction *helpDialogProgrammingFAQsB3;
    QAction *helpDialogProgrammingFAQsB4;
    QAction *helpDialogProgrammingFAQsC1;
    QAction *helpDialogProgrammingFAQsC2;
    QAction *helpDialogProgrammingFAQsC3;
    QAction *licenseAction;
    QAction *disclaimerAction;
    QAction *aboutVisuaAction;
    QAction *aboutQtAction;
    QAction *creditsAction;
    QList<QAction*> pluginActions;
    QList<QAction*> templateActions;

    // Menus
    QMenu *fileMenu;
    QMenu *stataMenu;
    QMenu *debugMenu;
    QMenu *pluginsMenu;
    QMenu *viewMenu;
    QMenu *helpMenu;

    // Stata
    QString stataExecutablePath;

    // Plugins
    QList<VisuaPluginInterface*> plugins;

    // Last directories
    QString lastDirForOpen;
    QString lastDirForSaveAs;
    QString lastDirForExport;

private slots:
    void onNewFileActionTriggered();
    void onOpenFileActionTriggered();
    bool onSaveFileActionTriggered();
    bool saveAs();
    void onExportActionTriggered();
    void onDataChanged(QModelIndex, QModelIndex);
    void onTestDialogInStataActionTriggered();
    void viewLastStataLog();
    void onStataPathTriggered();
    void onDebugActionTriggered();
    void onViewLastDebugReportActionTriggered();
    void onViewObjectListActionChecked(bool checked);
    void onViewObjectPropertiesActionChecked(bool checked);
    void onViewCodeChecked(bool checked);
    void onViewObjectBoxActionChecked(bool checked);
    void onHelpContentsActionTriggered();
    void onHelpDialogProgrammingTriggered();
    void onHelpDialogProgrammingFAQsTriggered();
    void onLicenseActionTriggered();
    void onDisclaimerActionTriggered();
    void onAboutVisuaActionTriggered();
    void onCreditsActionTriggered();
    void onErrorInStataProcess(QProcess::ProcessError processError);
    void onPluginTriggered();
    void onNewFromTemplateTriggered();
    void onWasAttemptedToInsertProhibitedObjInDialog();
    void onDataChangedInObjectPropertiesProxyModel(const QModelIndex &topLeft,
                                                   const QModelIndex &bottomRight);
    void prova();

protected:
    void closeEvent(QCloseEvent *event);
};

#endif // MAINWINDOW_H
