#include "objectpropertiesproxymodel.h"

#include <QSize>
#include "abstractobj.h"

#include <QDebug>

ObjectPropertiesProxyModel::ObjectPropertiesProxyModel(QObject *parent) :
    QAbstractProxyModel(parent)
{
    selectedPrimaryRow = -1;
    selectedSecondaryRow = -1;
}

QModelIndex ObjectPropertiesProxyModel::mapFromSource(const QModelIndex &sourceIndex) const
{
    if (!sourceIndex.isValid() || selectedPrimaryRow == -1)
        return QModelIndex();

    if (sourceIndex.column() == 0) // First column of source model has not a corresponding proxy index
        return QModelIndex();
    else
        return index(sourceIndex.column() - 1, 1, QModelIndex());
}

QModelIndex ObjectPropertiesProxyModel::mapToSource(const QModelIndex &proxyIndex) const
{
    if (!proxyIndex.isValid() || selectedPrimaryRow == -1)
        return QModelIndex();
    else
    {
        if (selectedSecondaryRow == -1)
            return sourceModel()->index(selectedPrimaryRow, proxyIndex.row() + 1, QModelIndex());
        else
        {
            QModelIndex sourceModelParentIndex =
                    sourceModel()->index(selectedPrimaryRow, 0, QModelIndex());
            return sourceModel()->index(selectedSecondaryRow, proxyIndex.row() + 1, sourceModelParentIndex);
        }
    }
}

QModelIndex ObjectPropertiesProxyModel::index(int row, int column, const QModelIndex &parent) const
{
    if (!hasIndex(row, column, parent) || selectedPrimaryRow == -1)
        return QModelIndex();

    if (column == 0)
    {
        AbstractObj* obj = static_cast<AbstractObj*>(index(row, 1, parent).internalPointer());
        return createIndex(row, 0, obj);
    }

    if (!parent.isValid() && column == 1)
    {
        if (selectedSecondaryRow == -1)
        {
            QModelIndex index = sourceModel()->index(selectedPrimaryRow, row + 1, QModelIndex());
            AbstractObj *obj = static_cast<AbstractObj*>(index.internalPointer());
            return createIndex(row, 1, obj);
        }
        else
        {
            QModelIndex sourceModelParentIndex = sourceModel()->index(selectedPrimaryRow, 0, QModelIndex());
            QModelIndex index = sourceModel()->index(selectedSecondaryRow, row + 1, sourceModelParentIndex);
            AbstractObj *obj = static_cast<AbstractObj*>(index.internalPointer());
            return createIndex(row, 1, obj);
        }
    }
    else
        return QModelIndex();
}

QModelIndex ObjectPropertiesProxyModel::parent(const QModelIndex &child) const
{
    Q_UNUSED(child)
    return QModelIndex();
}

int ObjectPropertiesProxyModel::rowCount(const QModelIndex &parent) const
{
    if (selectedPrimaryRow == -1)
        return 0;

    if (!parent.isValid())
    {
        if (selectedSecondaryRow == -1)
        {
            QModelIndex index = sourceModel()->index(selectedPrimaryRow, 0, QModelIndex());
            AbstractObj *obj = static_cast<AbstractObj*>(index.internalPointer());
            return obj->propertiesCount();
        }
        else
        {
            QModelIndex sourceModelParentIndex = sourceModel()->index(selectedPrimaryRow, 0, QModelIndex());
            QModelIndex index = sourceModel()->index(selectedSecondaryRow, 0, sourceModelParentIndex);
            AbstractObj *obj = static_cast<AbstractObj*>(index.internalPointer());
            return obj->propertiesCount();
        }
    }
    else
        return 0;
}

int ObjectPropertiesProxyModel::columnCount(const QModelIndex &parent) const
{
    return 2;
}

QVariant ObjectPropertiesProxyModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid() || selectedPrimaryRow == -1)
        return QVariant();

    switch (role)
    {
        case Qt::DisplayRole:
            switch (index.column())
            {
                case 0:
                    {
                    AbstractObj* obj = static_cast<AbstractObj*>(mapToSource(index).internalPointer());
                    return obj->propertyAt(index.row())->propertyNameForDisplayRoleData();
                    }
                    break;
                case 1:
                    return sourceModel()->data(mapToSource(index), Qt::DisplayRole);
                    break;
                default:
                    return QVariant();
                    break;
            }
            break;
        case Qt::EditRole:
            switch (index.column())
            {
                case 0: // No editing allowed for column 0
                    return QVariant();
                    break;
                case 1:
                    return sourceModel()->data(mapToSource(index), Qt::EditRole);
                    break;
                default:
                    break;
                    return QVariant();
            }
        case Qt::SizeHintRole:
            {
            AbstractObj *obj = static_cast<AbstractObj*>(mapToSource(index).internalPointer());
            switch (obj->objType())
            {
                case AbstractObj::Program:
                    if (index.row() == 1)
                        return QSize(50, 150);
                    else
                        return QSize(10, 20);
                    break;
                case AbstractObj::Script:
                    if (index.row() == 1)
                        return QSize(50, 150);
                    else
                        return QSize(10, 20);
                    break;
//                case AbstractObj::List:
//                    if (index.row() == 1)
//                        return QSize(50, 200);
//                    else
//                        return QSize(10, 20);
                default:
                    return QSize(10, 20);
                    break;
            }
            }
            break;        
        case Qt::TextAlignmentRole:
            {
            AbstractObj *obj = static_cast<AbstractObj*>(mapToSource(index).internalPointer());
            switch (obj->objType())
            {
                case AbstractObj::Script:
                    if (index.row() == 1)
                        return QVariant(Qt::AlignLeft | Qt::AlignTop);
                    else
                        return QVariant(Qt::AlignLeft | Qt::AlignVCenter);
                    break;
                case AbstractObj::Program:
                    if (index.row() == 1)
                        return QVariant(Qt::AlignLeft | Qt::AlignTop);
                    else
                        return QVariant(Qt::AlignLeft | Qt::AlignVCenter);
                    break;
                default:
                    return QVariant(Qt::AlignLeft | Qt::AlignVCenter);
                    break;
            }
            }
            break;
        default:
            return QVariant();
            break;
    }
}

QVariant ObjectPropertiesProxyModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (role != Qt::DisplayRole)
        return QVariant();

    if (orientation == Qt::Horizontal)
    {
        switch (section)
        {
            case 0:
                return "Property";
                break;
            case 1:
                return "Value";
                break;
            default:
                return QVariant();
                break;
        }
    }
    else
        return QVariant();
}

void ObjectPropertiesProxyModel::onSelectionChanged(QItemSelection selected, QItemSelection deselected)
{
    Q_UNUSED(deselected)
    if (selected.isEmpty())
    {
        selectedSecondaryRow = -1;
        reset();
        return;
    }

    if (selected.at(0).indexes().isEmpty())
    {
        selectedSecondaryRow = -1;
        reset();
        return;
    }
    QModelIndex index = selected.at(0).indexes().at(0);
    QModelIndex parentIndex = index.parent();
    int rowIndex = index.row();
    if (!parentIndex.isValid())
    {
        selectedPrimaryRow = rowIndex;
        selectedSecondaryRow = -1;
    }
    else
    {
        selectedPrimaryRow = parentIndex.row();
        selectedSecondaryRow = rowIndex;
    }

    reset();
}

void ObjectPropertiesProxyModel::onItemMovedUp()
{
    if (selectedPrimaryRow == -1) return;

    if (selectedSecondaryRow == -1)
        selectedPrimaryRow--;
    else
        selectedSecondaryRow--;

    reset();
}

void ObjectPropertiesProxyModel::onItemMovedDown()
{
    if (selectedPrimaryRow == -1) return;

    if (selectedSecondaryRow == -1)
        selectedPrimaryRow++;
    else
        selectedSecondaryRow++;

    reset();
}

Qt::ItemFlags ObjectPropertiesProxyModel::flags(const QModelIndex &index) const
{
    if (!index.isValid())
        return 0;

    switch (index.column())
    {
        case 0:
            return Qt::ItemIsEnabled | Qt::ItemIsSelectable;
            break;
        case 1:
            return Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsEditable;
            break;
        default:
            return 0;
            break;
    }
}

bool ObjectPropertiesProxyModel::setData(const QModelIndex &index, const QVariant &value, int role)
{
    if (index.isValid() && role == Qt::EditRole)
    {
        if (sourceModel()->setData(mapToSource(index), value, Qt::EditRole))
        {
            emit dataChanged(index, index);
            return true;
        }
        else
            return false;
    }
    else
        return false;
}

void ObjectPropertiesProxyModel::onMainModelWasResetted()
{
    selectedPrimaryRow = -1;
    selectedSecondaryRow = -1;
    reset();
}
